from typing import Optional

from pydantic import BaseModel, Field


class SipAuth(BaseModel):
    """Model representing the authentication details for the SIP INVITE request for HTTP
    digest authentication, if it is required by your SIP platform.

    Args:
        username (str): The username for HTTP digest authentication.
        password (str): The password for HTTP digest authentication.
    """

    username: str
    password: str


class SipOptions(BaseModel):
    """Model representing the SIP options for the call.

    Args:
        uri (str): The SIP URI to be used as the destination of the SIP call.
        from_ (Optional[str]): The number or string sent to the final SIP number
            as the caller. It must be a string in the form of `from@example.com`, where
            `from` can be a string or a number.
        headers (Optional[dict]): Custom headers to be added to the SIP INVITE request.
        auth (Optional[SipAuth]): Authentication details for the SIP INVITE request.
        secure (Optional[bool]): Indicates whether the media must be transmitted encrypted.
            Default is false.
        video (Optional[bool]): Indicates whether the SIP call will include video.
            Default is false.
        observe_force_mute (Optional[bool]): Indicates whether the SIP endpoint observes
            force mute moderation.
    """

    uri: str
    from_: Optional[str] = Field(None, serialization_alias='from')
    headers: Optional[dict] = None
    auth: Optional[SipAuth] = None
    secure: Optional[bool] = None
    video: Optional[bool] = None
    observe_force_mute: Optional[bool] = Field(
        None, serialization_alias='observeForceMute'
    )


class InitiateSipRequest(BaseModel):
    """Model representing the SIP options for joining a Vonage Video session.

    Args:
        session_id (str): The Vonage Video session ID for the SIP call to join.
        token (str): The Vonage Video token to be used for the participant being called.
        sip (Sip): The SIP options for the call.
    """

    session_id: str = Field(..., serialization_alias='sessionId')
    token: str
    sip: SipOptions


class SipCall(BaseModel):
    """Model representing the details of a SIP call.

    Args:
        id (str): A unique ID for the SIP call.
        project_id (str): The Vonage Video project ID for the SIP call.
        session_id (str): The Vonage Video session ID for the SIP call.
        connection_id (str): The Vonage Video connection ID for the SIP call's connection
            in the Vonage Video session.
        stream_id (str): The Vonage Video stream ID for the SIP call's stream in the
            Vonage Video session.
        created_at (int): The timestamp when the SIP call was created,in milliseconds since
            the Unix epoch.
        updated_at (int): The timestamp when the SIP call was last updated, in milliseconds
            since the Unix epoch.
    """

    id: Optional[str] = None
    project_id: Optional[str] = Field(None, validation_alias='projectId')
    session_id: Optional[str] = Field(None, validation_alias='sessionId')
    connection_id: str = Field(None, validation_alias='connectionId')
    stream_id: str = Field(None, validation_alias='streamId')
    created_at: Optional[int] = Field(None, validation_alias='createdAt')
    updated_at: Optional[int] = Field(None, validation_alias='updatedAt')



from vonage_utils.errors import VonageError


class VideoError(VonageError):
    """Indicates an error when using the Vonage Voice API."""


class InvalidRoleError(VideoError):
    """The specified role was invalid."""


class TokenExpiryError(VideoError):
    """The specified token expiry time was invalid."""


class SipError(VideoError):
    """Error related to usage of SIP calls."""


class NoAudioOrVideoError(VideoError):
    """Either an audio or video stream must be included."""


class IndividualArchivePropertyError(VideoError):
    """The property cannot be set for `archive_mode: 'individual'`."""


class LayoutStylesheetError(VideoError):
    """Error with the `stylesheet` property when setting a layout."""


class LayoutScreenshareTypeError(VideoError):
    """Error with the `screenshare_type` property when setting a layout."""


class InvalidArchiveStateError(VideoError):
    """The archive state was invalid for the specified operation."""


class InvalidHlsOptionsError(VideoError):
    """The HLS options were invalid."""


class InvalidOutputOptionsError(VideoError):
    """The output options were invalid."""


class InvalidBroadcastStateError(VideoError):
    """The broadcast state was invalid for the specified operation."""


class RoutedSessionRequiredError(VideoError):
    """The operation requires a session with `media_mode=routed`."""



from typing import Optional

from pydantic import BaseModel, Field, model_validator

from .enums import ArchiveMode, MediaMode, P2pPreference


class SessionOptions(BaseModel):
    """Options for creating a new session.

    Args:
        media_mode (MediaMode): The media mode for the session.
        archive_mode (ArchiveMode): The archive mode for the session.
        location (str): The location of the session.
        e2ee (bool): Whether end-to-end encryption is enabled.
        p2p_preference (str): The preference for peer-to-peer connections.
            This is set automatically by selecting the `media_mode`.
    """

    archive_mode: Optional[ArchiveMode] = Field(None, serialization_alias='archiveMode')
    location: Optional[str] = None
    media_mode: Optional[MediaMode] = None
    e2ee: Optional[bool] = None
    p2p_preference: Optional[str] = Field(
        P2pPreference.DISABLED, serialization_alias='p2p.preference'
    )

    @model_validator(mode='after')
    def set_p2p_preference(self):
        if self.media_mode == MediaMode.ROUTED:
            self.p2p_preference = P2pPreference.DISABLED
        if self.media_mode == MediaMode.RELAYED:
            self.p2p_preference = P2pPreference.ALWAYS
        return self

    @model_validator(mode='after')
    def set_p2p_preference_if_archive_mode_set(self):
        if self.archive_mode == ArchiveMode.ALWAYS:
            self.p2p_preference = P2pPreference.DISABLED
        return self


class VideoSession(BaseModel):
    """The new session ID and options specified in the request.

    Args:
        session_id (str): The session ID.
        archive_mode (ArchiveMode, Optional): The archive mode for the session.
        media_mode (MediaMode, Optional): The media mode for the session.
        location (str, Optional): The location of the session.
        e2ee (bool, Optional): Whether end-to-end encryption is enabled for the session.
    """

    session_id: str
    archive_mode: Optional[ArchiveMode] = None
    media_mode: Optional[MediaMode] = None
    location: Optional[str] = None
    e2ee: Optional[bool] = None



from typing import Optional

from pydantic import BaseModel, Field

from .enums import LanguageCode


class CaptionsOptions(BaseModel):
    """The Options to send captions.

    Args:
        session_id (str): The session ID.
        token (str): A valid token with moderation privileges.
        language_code (LanguageCode, Optional): The language code.
        max_duration (int, Optional): The maximum duration.
        partial_captions (bool, Optional): The partial captions.
        status_callback_url (str, Optional): The status callback URL.
    """

    session_id: str = Field(..., serialization_alias='sessionId')
    token: str
    language_code: Optional[LanguageCode] = Field(
        None, serialization_alias='languageCode'
    )
    max_duration: Optional[int] = Field(
        None, ge=300, le=14400, serialization_alias='maxDuration'
    )
    partial_captions: Optional[bool] = Field(None, serialization_alias='partialCaptions')
    status_callback_url: Optional[str] = Field(
        None, min_length=15, max_length=2048, serialization_alias='statusCallbackUrl'
    )


class CaptionsData(BaseModel):
    """Class containing captions ID.

    Args:
        captions_id (str): The captions ID.
    """

    captions_id: str = Field(..., serialization_alias='captionsId')
